package async.client;

import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import javax.xml.namespace.QName;
import javax.xml.ws.AsyncHandler;
import javax.xml.ws.Response;

public class StudentsClient {
	private final static QName STUDENTS_QNAME = new QName("http://asu.dgtu.donetsk.ua/ex/students", "Students");

	class StudentsAsyncHandler implements AsyncHandler<GetStudentListResponse> {
		private GetStudentListResponse response;

		@Override
		public void handleResponse(Response<GetStudentListResponse> res) {
			try {
				response = res.get(2000, TimeUnit.MILLISECONDS);

			} catch (Exception ex) {
				ex.printStackTrace();
			}
		}

		public List<Student> getResponse() {
			return response.getReturn();
		}
	};

	private static URL getWSDLURL(String urlStr) {
		URL url = null;
		try {
			url = new URL(urlStr);
		} catch (MalformedURLException e) {
			e.printStackTrace();
			throw new RuntimeException(e);
		}
		return url;
	}

	public void processCallback(URL url) {
		Students svc = new Students(url, STUDENTS_QNAME);
		StudentService port = svc.getStudentsPort();
		StudentsAsyncHandler asyncHandler = new StudentsAsyncHandler();
		Future<?> futureResponse = port.getStudentListAsync(asyncHandler);

		while (!futureResponse.isDone()) {
			try {
				System.out.println("Waiting...");
				Thread.sleep(10000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		List<Student> students = asyncHandler.getResponse();
		printStudentList(students);
	}

	private static void printStudentList(List<Student> list) {
		for (Student s : list) {
			StringBuilder sb = new StringBuilder();
			sb.append(s.lastName);
			sb.append(" ");
			sb.append(s.firstName);
			sb.append(" ");
			sb.append(s.group);
			sb.append(" ");
			sb.append(s.transcriptNum);
			System.out.println(sb);
		}
	}

	public void processPolling(URL url) {
		Students svc = new Students(url, STUDENTS_QNAME);
		StudentService port = svc.getStudentsPort();
		Response<GetStudentListResponse> response = port.getStudentListAsync();
		while(!response.isDone()) {
			try {
				System.out.println("Polling...");
				Thread.sleep(10000);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		try {
			List<Student> students = response.get().getReturn();
			printStudentList(students);
		} catch (InterruptedException e) {
			e.printStackTrace();
		} catch (ExecutionException e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args) {
		URL url = getWSDLURL("http://localhost:82/Students?wsdl");
		StudentsClient client = new StudentsClient();
		client.processCallback(url);
		client.processPolling(url);
	}

}
